//go:generate easyjson -all timesheet_row.go

package renderModels

import (
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	"github.com/xaionaro-go/extime"
)

type TimesheetRow struct {
	Date          extime.Date
	PersonId      int
	EmpGUID       int
	HoursRequired *float32
	StfCardId     int
	TimeUseCode   string

	Formular *Formular `json:",omitempty"`

	Person *Person `json:",omitempty"`
}

func NewTimesheetRow(in asuModels.TimesheetRow) (out TimesheetRow) {
	out = TimesheetRow{
		Date:          in.ADate,
		PersonId:      in.PersonId,
		EmpGUID:       in.EmpGUID,
		HoursRequired: in.Hours,
		StfCardId:     in.StfCardId,
		TimeUseCode:   in.TimeUseCode,
	}

	if in.IsFormularReady() {
		formular := NewFormular(in.GetFormular())
		out.Formular = &formular
	}

	if in.IsPersonReady() {
		person := NewPersonFromASU(in.GetPerson())
		out.Person = &person
	}

	return
}

type TimesheetRows []TimesheetRow

func NewTimesheetRows(in asuModels.TimesheetRows) (out TimesheetRows) {
	for _, inTimesheetRow := range in {
		out = append(out, NewTimesheetRow(inTimesheetRow))
	}

	return
}
