package renderModels

import (
	"database/sql"
	asuModels "devel.mephi.ru/dyokunev/go-asu-models"
	"fmt"
)

type Unit struct {
	asuModels.Unit
	Formulars             Formulars
	HasFormularsRecursive *bool

	children      []*Unit `reform:"-"`
	childrenReady bool    `reform:"-"`

	Children *[]*Unit
}

func NewUnit(in asuModels.Unit) (out Unit) {
	out.Unit = in

	if in.IsFormularsReady() {
		out.Formulars = NewFormulars(in.GetFormulars())
	}

	return
}

type Units []Unit

func (units Units) GetUnitIds() (unitIds []int) {
	for _, unit := range units {
		unitIds = append(unitIds, unit.Id)
	}

	return
}

func (units Units) GetPersNumbers() (unitIds []int) {
	for _, unit := range units {
		if unit.PersNumber == nil {
			continue
		}
		unitIds = append(unitIds, *unit.PersNumber)
	}

	return
}

func (units Units) ToMap() map[int]*Unit {
	unitMap := map[int]*Unit{}

	for idx, unit := range units {
		unitMap[unit.Id] = &units[idx]
	}

	return unitMap
}

func (units Units) ToPersNumberMap() map[int]*Unit {
	unitMap := map[int]*Unit{}

	for idx, unit := range units {
		if unit.PersNumber == nil {
			continue
		}
		unitMap[*unit.PersNumber] = &units[idx]
	}

	return unitMap
}

func (units Units) calculateTree(strict bool) {
	unitMap := units.ToMap()

	for idx := range units {
		unit := &units[idx]

		if unit.ParentId == nil {
			continue
		}

		parent := unitMap[*unit.ParentId]

		if parent == nil {
			if strict {
				panic(fmt.Sprintf("Not full units list. len(units) == %v; *unit.ParentId == %v", units, *unit.ParentId))
			}
			continue
		}

		parent.children = append(parent.children, unit)
	}

	for idx := range units {
		units[idx].childrenReady = true
	}
}
func (units Units) CalculateTree() Units {
	units.calculateTree(true)
	return units
}
func (unit Unit) IsChildrenReady() bool {
	return unit.childrenReady
}

func NewUnits(in asuModels.Units) (out Units) {
	for _, inUnit := range in {
		out = append(out, NewUnit(inUnit))
	}

	return
}

func (u Unit) GetChildrenPtrs() []*Unit {
	if !u.childrenReady {
		panic("unit is not ready for method GetChildrenPtrs: it's required to call CalculateTree method, first")
	}
	return u.children
}

func (unit *Unit) DoRecursive(f func(*Unit, interface{}) bool, arg interface{}) bool {
	if !f(unit, arg) {
		return false
	}

	for _, child := range unit.GetChildrenPtrs() {
		if !child.DoRecursive(f, arg) {
			return false
		}
	}

	return true
}

func (units Units) DoRecursive(f func(*Unit, interface{}) bool, arg interface{}) bool {
	for _, unit := range units {
		unit.DoRecursive(f, arg)
	}

	return true
}

func (units Units) SetIfHasFormularsRecursive() Units {
	if len(units) == 0 {
		return units
	}
	if !units[0].IsChildrenReady() {
		panic("The tree is not calculated, yet (use CalculateTree method)")
	}

	unitMap := units.ToPersNumberMap()

	persNumbers := units.GetPersNumbers()

	formulars, err := asuModels.FormularSQL.ActiveOnly().Group("PersNumber").Select("PersNumber IN (?)", persNumbers)
	if err != nil && err != sql.ErrNoRows {
		panic(err)
	}

	trueValue := true
	for _, formular := range formulars {
		if formular.PersNumber == nil {
			continue
		}
		unitMap[*formular.PersNumber].HasFormularsRecursive = &trueValue
	}

	//units.PrepareChildrenTree()

	falseValue := false
	for idx := range units {
		if units[idx].HasFormularsRecursive != nil {
			continue
		}
		units[idx].DoRecursive(func(unit *Unit, arg interface{}) bool {
			if unit.HasFormularsRecursive != nil {
				if *unit.HasFormularsRecursive == true {
					units[idx].HasFormularsRecursive = unit.HasFormularsRecursive
					return false
				}
			}
			return true
		}, nil)
		if units[idx].HasFormularsRecursive == nil {
			units[idx].HasFormularsRecursive = &falseValue
		}
	}

	return units
}

func (unit *Unit) ShowChildren() *Unit {
	unit.Children = &unit.children
	return unit
}

func (units Units) ShowChildren() Units {
	for idx := range units {
		units[idx].ShowChildren()
	}

	return units
}

func (u *Unit) prepareChildrenTree(recursive bool) *Unit {
	if u.childrenReady {
		return u
	}

	asuChildren, err := asuModels.UnitSQL.Select(asuModels.Unit{ParentId: &u.Id})
	if err == sql.ErrNoRows {
		u.childrenReady = true
		return u
	}
	if err != nil {
		panic(err)
	}

	for _, asuChild := range asuChildren {
		child := NewUnit(asuChild)
		if recursive {
			child.PrepareChildrenTree()
		}

		u.children = append(u.children, &child)
	}

	u.childrenReady = true

	return u
}

func (u *Unit) PrepareChildrenTree() *Unit {
	return u.prepareChildrenTree(true)
}
func (u *Unit) PrepareChildren() *Unit {
	return u.prepareChildrenTree(false)
}

func (units Units) PrepareChildrenTree() Units {
	for idx := range units {
		units[idx].PrepareChildrenTree()
	}

	return units
}
func (units Units) PrepareChildren() Units {
	for idx := range units {
		units[idx].PrepareChildren()
	}

	return units
}
